--=========================================================
--This is how many days of activity are stored in cache:
--=========================================================
SELECT DATEDIFF(dd,[create_date], GETDATE()) FROM sys.databases WHERE name = 'tempdb';


--=========================================================
--Return usage results for all indexes in current database
--=========================================================
SELECT OBJECT_NAME(ixU.[object_id]) AS object__name
	, sI.indid AS index_id
	, sI.name AS index_name
	, sI.rowcnt AS row_count
	, ixU.user_updates
	, ixU.user_seeks
	, ixU.user_scans
	, ixU.user_lookups
	, ixU.[system_updates]
	, ixU.[system_seeks]
	, ixU.system_scans
	, ixU.system_lookups
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.sysindexes AS sI 
		ON ixU.[object_id] = sI.id AND ixU.index_id = sI.indid 
WHERE ixU.database_id = db_id()
	AND sI.rowcnt > 500
ORDER BY 1, 2;
GO


--===================================================================
--Return results for all indexes not being read, but are written to
--===================================================================
SELECT OBJECT_NAME(ixU.[object_id]) AS object__name
	, sI.indid AS index_id
	, sI.name AS index_name
	, sI.rowcnt AS row_count
	, ixU.user_updates
--	, ixU.user_seeks + ixU.user_scans + ixU.user_lookups AS [user_reads]
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.sysindexes AS sI ON ixU.[object_id] = sI.id AND ixU.index_id = sI.indid 
WHERE ixU.database_id = db_id() 
	AND ixU.index_id > 0
	AND (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) = 0 
	AND ixU.user_updates > 0 
ORDER BY 1, 2;
GO


--=========================================================
--Include generation of drop command
--=========================================================
SELECT 
	'['+ DB_NAME() + '].[' + SSU.name + '].[' + SO.name + ']' AS [fully_qualified_object_name]
	, I.index_id
	, I.name AS [index_name]
	, SUM(P.rows) AS [total_rows]
	--, (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS [user_reads] 
	, ixU.user_updates AS [user_writes]
	, 'DROP INDEX [' + I.name + '] ON [' + SSU.name + '].[' + SO.name + '] WITH ( ONLINE = OFF )' AS [drop_command]
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.indexes AS I 
		ON ixU.OBJECT_ID = I.OBJECT_ID 
			AND I.index_id = ixU.index_id   
	INNER JOIN sys.partitions AS P 
		ON ixU.OBJECT_ID = P.OBJECT_ID 
			AND P.index_id= ixU.index_id 
	INNER JOIN sys.objects SO 
		ON ixU.[object_id] = SO.[object_id] 
	INNER JOIN sys.sysusers SSU 
		ON SO.[schema_id] = SSU.UID
WHERE 
	OBJECTPROPERTY(ixU.OBJECT_ID,'IsUserTable') = 1 
	AND ixU.index_id > 0 
	AND ixU.database_id = DB_ID()
GROUP BY I.name
	, I.index_id 
	, (ixU.user_seeks + ixU.user_scans + ixU.user_lookups)
	, user_updates 
	,SSU.name
	, SO.name
HAVING 
	(ixU.user_seeks + ixU.user_scans + ixU.user_lookups) = 0
ORDER BY SO.name, I.index_id;
GO


--=========================================================
--Analyze Usage Via Read :: Write Ratio
--=========================================================
SELECT OBJECT_NAME(ixU.[object_id]) AS object__name
	, sI.indid AS index_id
	, sI.name AS index_name
	, sI.rowcnt AS row_count
	, (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS [user_reads]
	, ixU.user_updates
	, (1.0) * (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) / ixU.user_updates AS [reads_per_write]
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.sysindexes AS sI ON ixU.[object_id] = sI.id AND ixU.index_id = sI.indid 
WHERE ixU.database_id = db_id() 
	AND ixU.index_id > 0
	AND ixU.user_updates > 0
	AND sI.rowcnt > 0
ORDER BY 7 ASC, ixU.user_updates DESC, sI.rowcnt DESC, 1, 2;
GO



--===========================================================
--Analyze for Possible Poor Clustering Key Choice
--=========================================================
WITH grouped_reads AS
	(
		SELECT ixU.[object_id]
			, SUM(ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS total_user_reads
			, SUM(ixU.user_updates) AS total_user_writes
		FROM sys.dm_db_index_usage_stats AS ixU 
		WHERE ixU.database_id = db_id() 
			AND ixU.index_id > 0
			AND ixU.user_updates > 0	
		GROUP BY ixU.[object_id]
		HAVING SUM(ixU.user_seeks + ixU.user_scans + ixU.user_lookups) > 0
			AND SUM(ixU.user_updates) > 0
	)

SELECT OBJECT_NAME(ixU.[object_id]) AS object__name
	, sI.indid AS index_id
	, sI.name AS index_name
	, sI.rowcnt AS row_count
	, (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS user_reads
	, GR.total_user_reads AS total_reads_for_all_indexes
	, CAST(100 * ((ixU.user_seeks + ixU.user_scans + ixU.user_lookups) / (GR.total_user_reads * 1.0)) AS DECIMAL(4,1)) AS pct_total_reads
	, ixU.user_updates
	, GR.total_user_writes AS total_writes_for_all_indexes
	, CAST((100 * ixU.user_updates / (GR.total_user_writes) * 1.0) AS DECIMAL(4,1)) AS pct_total_updates
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.sysindexes AS sI ON ixU.[object_id] = sI.id AND ixU.index_id = sI.indid 
	INNER JOIN grouped_reads AS GR ON GR.[object_id] = ixU.[object_id] 
WHERE ixU.database_id = db_id() 
	AND ixU.index_id > 0
	AND ixU.user_updates > 0
	AND sI.rowcnt > 500
ORDER BY 1, 2;
GO