--============================================================================
-- INDEX REBUILD COUNT LAST 31 DAYS
--============================================================================
CREATE TABLE ##Index_History
	(
		the_database sysname,
		the_schema sysname,
		the_object sysname,
		the_index sysname,
		index_type VARCHAR(13),
		fill_factor tinyint,
		rebuild_count int NULL,
		reorg_count int NULL
	);

CREATE TABLE ##Indexes
	(
		the_schema sysname,
		the_object sysname,
		the_index sysname,
		fill_factor tinyint
	);

INSERT INTO ##Indexes
	(
	the_schema
	, the_object
	, the_index
	, fill_factor
	)

SELECT OBJECT_SCHEMA_NAME(SO.object_id, db_id('ClientVantage')) AS the_schema
	, SO.name AS the_object
	, SI.name AS the_index
	, SI.fill_factor
FROM ClientVantage.sys.objects SO
	INNER JOIN ClientVantage.sys.indexes SI
		ON SO.object_id = SI.object_id
WHERE SI.index_id > 0;

INSERT INTO ##Index_History
         (the_database
        , the_schema
        , the_object
        , the_index
        , index_type
        , fill_factor
        , rebuild_count
        , reorg_count
         )

SELECT C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , CASE C.IndexType
		WHEN 1 THEN 'Clustered'
		ELSE 'Non-Clustered'
	END AS IndexType
	, IX.fill_factor
	, COUNT(C.ID) AS rebuild_count
	, 0 AS reorg_count
FROM iDBA.dbo.CommandLog C 
	LEFT JOIN ##Indexes IX ON C.SchemaName = IX.the_schema
		AND C.ObjectName = IX.the_object
		AND C.IndexName = IX.the_index
WHERE C.CommandType = 'ALTER_INDEX'
	AND C.ObjectType = 'U'
	AND C.Command LIKE '%REBUILD%'
	AND C.DatabaseName = 'ClientVantage'
GROUP BY C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , IndexType
	, IX.fill_factor;


--============================================================================
-- LAST INDEX REBUILD FOR A SPECIFIC INDEX
--============================================================================
WITH rebuild_history AS
	(
		SELECT ROW_NUMBER() OVER( PARTITION BY CL.ObjectName, CL.IndexName ORDER BY (CL.EndTime) DESC) AS rn
			, CL.ObjectName
			, CL.indexName
			, CL.EndTime
		FROM iDBA.[dbo].[CommandLog] AS CL 
		WHERE CL.DatabaseName = 'ClientVantage' 
			AND CL.Command LIKE '%REBUILD%' 
			AND CL.ObjectType = 'U'
	)

SELECT * FROM rebuild_history 
WHERE rn = 1 
ORDER BY ObjectName, IndexName	


--============================================================================
-- INDEX REORGANIZE (ONLY) COUNT LAST 31 DAYS
--============================================================================
INSERT INTO ##Index_History
         (the_database
        , the_schema
        , the_object
        , the_index
        , index_type
        , fill_factor
        , rebuild_count
        , reorg_count
         )

SELECT C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , CASE C.IndexType
		WHEN 1 THEN 'Clustered'
		ELSE 'Non-Clustered'
	END AS IndexType
	, IX.fill_factor
	, 0 AS rebuild_count
	, COUNT(C.ID) AS reorg__count
FROM iDBA.dbo.CommandLog C 
	LEFT JOIN ##Indexes IX ON C.SchemaName = IX.the_schema
		AND C.ObjectName = IX.the_object
		AND C.IndexName = IX.the_index
	LEFT JOIN ##Index_History IH
		ON C.DatabaseName = IH.the_database
			AND C.SchemaName = IH.the_schema
			AND C.ObjectName = IH.the_object
			AND C.IndexName = IH.the_index
WHERE C.CommandType = 'ALTER_INDEX'
	AND C.ObjectType = 'U'
	AND C.Command LIKE '%REORGANIZE%'
	AND C.DatabaseName = 'ClientVantage'
	AND IH.the_database IS NULL
GROUP BY C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , IndexType
	, IX.fill_factor;

--========================================================
-- ACCOUNT FOR INDEXES BOTH REBUILT AND REORGANIZED
--========================================================
UPDATE ##Index_History
SET reorg_count = C2.reorganize_count
FROM ##Index_History IH
	INNER JOIN
		(
			SELECT C.DatabaseName
				, C.SchemaName
				, C.ObjectName
				, C.IndexName
				, COUNT(C.ID) AS reorganize_count
				 FROM iDBA.dbo.CommandLog C	
			WHERE C.CommandType = 'ALTER_INDEX'
				AND C.ObjectType = 'U'
				AND C.Command LIKE '%REORGANIZE%'
				AND C.DatabaseName = 'ClientVantage'
			GROUP BY C.DatabaseName
				, C.SchemaName
				, C.ObjectName
				, C.IndexName
		) C2 ON IH.the_database = C2.DatabaseName
			AND IH.the_schema = C2.SchemaName 
			AND IH.the_object = C2.ObjectName   
			AND IH.the_index = C2.IndexName 
WHERE IH.rebuild_count > 0


--============================================================================
-- RETURN THE RESULTS
--============================================================================
SELECT the_database
    , the_schema
    , the_object
    , the_index
    , index_type
    , fill_factor
    , rebuild_count
    , reorg_count 
FROM ##Index_History 
ORDER BY the_database
	, rebuild_count DESC
	, reorg_count DESC
	, the_object
	, the_index

--============================================================================
-- CLEANUP THE MESS
--============================================================================
IF EXISTS (SELECT name FROM tempdb.sys.objects WHERE name = '##Index_History')
BEGIN
	DROP TABLE ##Index_History
END

IF EXISTS (SELECT name FROM tempdb.sys.objects WHERE name = '##Indexes')
BEGIN
	DROP TABLE ##Indexes
END

